.BG
.FN SchurTest
.TL
Test if autoregressive operator has all roots outside unit circle
.DN
For a vector argument phi, let p=length(phi). Then the test is
whether or not all roots of the polynomial equation 
(1 - phi[1]B - ... -phi[p]B^p = 0) are outside the unit circle.
If phi represents the coefficients in an autoregression,
z[t] - phi[1]z[t-1] - ... - phi[p]z[t-p] = a[t] then this
is equivalent to testing if it is causal in the sense of 
Brockwell & Davis (1991). This condition is necessary and
sufficient. Of course, if phi represents the coefficients in
a moving average process, 
z[t] = a[t] - phi[1]a[t-1] -...- phi[p] a[t-p],
then this is a necessary and sufficient condition for
stationarity.
.CS
SchurTest(phi)
.RA
.AG phi
a vector giving the coefficients in the autoregressive operator
.RT
T or F according as the roots are outside the unit circle or not
.SE

.DT
The details of this method were first derived by Schur in a paper
in 1917 in German. The application to autoregressive models was
first pointed out by Pagano (1973). The concept of causal models
was introduced by Brocwell & Davis.

Some insight into how close the model is to the admissible boundary
can be obtained by looking at sum(phi). If sum(phi) is close to
1.0 then this means that (1-B) nearly be a factor of the autoregressive
polynomial. One can also use the S function polyroot.


.SH REFERENCES
 Pagano, M. (1973),
 When is an autoregressive process stationary? 
 Communications in Statistics, A, 1, 533--544.

 Brockwell & Davis (1991),
 Time Series: Theory and Methods. Springer-Verlag.

.SA
polyroot

.EX
SchurTest(0.9)
SchurTest(c(0.9, 0.9))
.KW time series; arma
