"SchurTest"<-
function(phi)
{
####test if all roots of 1-phi[1] B - ... - phi[length(phi)] B^length(phi) = 0 are outside the unit circle#######
#Reference: Pagano, M. (1973) When is an autoregressive process stationary? Commun. Stat.
	if(length(phi) == 0) {
		return(T)
	}
	p <- length(phi)
	phi <- c(-1, phi)
	A <- matrix(numeric(p^2), nrow = p, ncol = p)
	for(j in 1:p) {
		for(i in 1:p) {
			if(j > i) {
				A[i, j] <- A[j, i]
			}
			else {
				k <- 1:min(i, j)
				A[i, j] <- sum(phi[1 + i - k] * phi[1 + j - k] - 
				  phi[1 + p + k - i] * phi[1 + p + k - j])
			}
		}
	}
	if(dim(A)[1] == attr(chol(A, pivot = T), "rank")) {
		return(T)
	}
	return(F)
}
"Toeplitz"<-
function(x)
{
	matrix(x[1 + abs(outer(seq(along = x), seq(along = x), FUN = "-"))], 
		byrow = T, ncol = length(x))
}
"iARMA"<-
function(phi = numeric(0), theta = numeric(0))
{
########information matrix of arma#####################
	if(!(SchurTest(phi) & SchurTest(theta))) {
		cat("Model is non-causal or non-invertible\n")
		return(NULL)
	}
	p <- length(phi)
	q <- length(theta)
	unames <- character(0)
	vnames <- character(0)
	if(p > 0) {
		if(p > 1) {
			vvmatrix <- (tccfAR(phi, phi)[ - (1:(p - 1))])[ - (p + 
				1)]
		}
		else if(p == 1) {
			vvmatrix <- tccfAR(phi, phi)[ - (p + 1)]
		}
		vvmatrix <- Toeplitz(vvmatrix)
		imatrix <- vvmatrix
		vnames <- paste("phi(", 1:p, ")", sep = "")
	}
	if(q > 0) {
		if(q > 1) {
			uumatrix <- (tccfAR(theta, theta)[ - (1:(q - 1))])[ - (
				q + 1)]
		}
		else if(q == 1) {
			uumatrix <- tccfAR(theta, theta)[ - (q + 1)]
		}
		uumatrix <- Toeplitz(uumatrix)
		imatrix <- uumatrix
		unames <- paste("theta(", 1:q, ")", sep = "")
	}
	if(p > 0 && q > 0) {
		uvmatrix <- matrix(numeric(1), nrow = p, ncol = q)
		tuv <-  - tccfAR(phi, theta)
		for(i in 1:p) {
			for(j in 1:q) {
				uvmatrix[i, j] <- tuv[q + i - j]
			}
		}
		imatrix <- cbind(rbind(vvmatrix, t(uvmatrix)), rbind(uvmatrix, 
			uumatrix))
	}
	inames <- c(vnames, unames)
	dimnames(imatrix) <- list(inames, inames)
	return(imatrix)
}
"iFARMA"<-
function(phi = numeric(0), theta = numeric(0))
{
#information matrix of fractional ARMA
	if(!(SchurTest(phi) & SchurTest(theta))) {
		cat("Model is non-causal or non-invertible\n")
		return(NULL)
	}
	I22 <- (pi^2)/6
	if((length(phi) == 0) && (length(theta) == 0))
		return(I22)
	I11 <- iARMA(phi = phi, theta = theta)
	J11 <- numeric(0)
	if(length(phi) > 0)
		J11 <-  - jFARMA(phi)
	J12 <- numeric(0)
	if(length(theta) > 0)
		J12 <- jFARMA(theta)
	J <- c(J11, J12)
	I <- rbind(I11, J)
	J <- c(J, I22)
	I <- cbind(I, J)
	inames <- c(dimnames(I11)[[1]], "d")
	dimnames(I) <- list(inames, inames)
	return(I)
}
"jFARMA"<-
function(theta)
{
	maxlag <- 128
	psis <- psiwtsAR(theta, maxlag = maxlag)
	q <- length(theta)
	J <- numeric(q)
	for(k in 1:q) {
		J[k] <- sum(psis/(k + 0:maxlag))
	}
	return(J)
}
"psiwtsAR"<-
function(phi, maxlag)
{
	p <- length(phi)
	x <- numeric(maxlag + 1)
	x <- 1
	for(i in 1:p) {
		x[i + 1] <- crossprod(phi[1:i], (rev(x))[1:i])
	}
	if(maxlag > p) {
		for(i in (p + 1):maxlag) {
			x[i + 1] <- crossprod(phi, (rev(x))[1:p])
		}
	}
	return(x)
}
"psiwtsARMA"<-
function(phi = numeric(0), theta = numeric(0), maxlag = 20)
{
	r <- max((p <- length(phi)), (q <- length(theta)))
	phi2 <- theta2 <- numeric(r)
	maxlagp1 <- 1 + maxlag
	if(q > 0)
		theta2[1:q] <- theta
	if(p > 0)
		phi2[1:p] <- phi
	x <- numeric(maxlagp1)
	x[1] <- 1
	if(r == 0)
		return(x[1:maxlagp1])
	for(i in 1:r) {
		x[i + 1] <- crossprod(phi2[1:i], rev(x[1:i])) - theta2[i]
	}
	if(p == 0)
		return(x[1:maxlagp1])
	if(maxlag > r) {
		for(i in (r + 1):maxlag) {
			x[i + 1] <- crossprod(phi, x[i - (0:(p - 1))])
		}
	}
	return(x[1:maxlagp1])
}
"tacvfARMA"<-
function(phi = numeric(0), theta = numeric(0), maxlag = 20)
{
#theoretical autocovariance function of arma#
#Reference: A.I. McLeod (1975) "Derivation of the theoretical autocovariance function of autoregressive-moving
# average models", Applied Statistics, Vol. 24. pp.255-256. 
	if(!(SchurTest(phi) & SchurTest(theta))) {
		cat("Model is non-causal or non-invertible\n")
		return(NULL)
	}
	p <- length(phi)
	q <- length(theta)
	maxlagp1 <- maxlag + 1
	if(max(p, q) == 0) {
		return(c(1, numeric(maxlagp1)))
	}
	r <- max(p, q) + 1
	b <- numeric(r)
	C <- numeric(q + 1)
	C[1] <- 1
	theta2 <- c(-1, theta)
	phi2 <- numeric(3 * r)
	phi2[r] <- -1
	if(p > 0) {
		phi2[r + 1:p] <- phi
	}
	if(q > 0) {
		for(k in 1:q) {
			C[k + 1] <-  - theta[k]
			if(p > 0) {
				for(i in 1:min(p, k)) {
				  C[k + 1] <- C[k + 1] + phi[i] * C[k + 1 - i]
				}
			}
		}
	}
	for(k in 0:q) {
		for(i in k:q) {
			b[k + 1] <- b[k + 1] - theta2[i + 1] * C[i - k + 1]
		}
	}
	if(p == 0) {
		g <- c(b, numeric(maxlagp1))[1:maxlagp1]
		return(g)
	}
	else if(p > 0) {
		a <- matrix(numeric(r^2), ncol = r)
		for(i in 1:r) {
			for(j in 1:r) {
				if(j == 1) {
				  a[i, j] <- phi2[r + i - 1]
				}
				else if(j != 1) {
				  a[i, j] <- phi2[r + i - j] + phi2[r + i + j - 
				    2]
				}
			}
		}
		g <- solve(a,  - b)
		if(length(g) <= maxlag) {
			g <- c(g, numeric(maxlag - r))
			for(i in (r + 1):maxlagp1) {
				g[i] <- phi %*% g[i - 1:p]
			}
			return(g)
		}
		else if(length(g) >= maxlagp1) {
			return(g[1:maxlagp1])
		}
	}
}
"tccfAR"<-
function(phi, theta)
{
#auxilary function used with iarma#########
#computes the theoretical cross-covariance function of two autoregressions
# z[t]-phi[1] z_[t-1] --- phi[p] z[t-p]     = a[t]
# z[t]-theta[1] z_[t-1] --- theta[q] z[t-q] = a[t]
# where p, q are length(phi), length(theta)
	p <- length(phi)
	q <- length(theta)
	if(p == 0 || q == 0)
		return(numeric(0))
	k <- p + q
	rhs <- c(-1, rep(0, k - 1))
	A <- matrix(numeric(k^2), nrow = k, ncol = k)
	for(i in 1:k) {
		for(j in 1:k) {
			imj <- i - j
			ijq <- i + j - q - 1
			if(i > q) {
				if(i > j && imj <= q)
				  A[i, j] <- theta[imj]
				else if(i > q && imj == 0)
				  A[i, j] <- -1
			}
			else {
				if(ijq > 0 && ijq <= p)
				  A[i, j] <- phi[ijq]
				else if(ijq == 0)
				  A[i, j] <- -1
			}
		}
	}
	return(solve(A, rhs))
}
"allfuncs"<-
c("SchurTest", "Toeplitz", "iARMA", "iFARMA", "jFARMA", "psiwtsAR", 
	"psiwtsARMA", "tacvfARMA", "tccfAR", "allfuncs")
